/*! @file main.c
 * @brief The main.c file of the Fixed packet length TX demo for Si4x55 devices.
 *
 * Contains the initialization of the MCU & the radio.
 * @n The main loop controls the program flow & the radio.
 *
 * @b CREATED
 * @n Silicon Laboratories Hungary Ltd.
 *
 * @b COPYRIGHT
 * @n Silicon Laboratories Confidential
 * @n Copyright 2012 Silicon Laboratories, Inc.
 * @n http://www.silabs.com
 *
 */

#include "..\drivers\bsp.h"
#include "deglitch_retime.h"
#include "phandler.h"

/*------------------------------------------------------------------------*/
/*                            Local Macros                                */
/*------------------------------------------------------------------------*/
#define PACKET_SEND_INTERVAL  250u

/*------------------------------------------------------------------------*/
/*                          Local variables                               */
/*------------------------------------------------------------------------*/
SEGMENT_VARIABLE(lPer_SecCntr, U16, SEG_XDATA);  /**< Demo sec time holder */
SEGMENT_VARIABLE(wwTmr_Tmr3Periode , UU16, SEG_XDATA); /**< Timer 3 period */

U32 xdata lWaitCnt;
/*------------------------------------------------------------------------*/
/*                      Local function prototypes                         */
/*------------------------------------------------------------------------*/
void vPlf_McuInit        (void);
void vInitializeHW       (void);
void DemoApp_Pollhandler (void);

/*------------------------------------------------------------------------*/
/*                          Function implementations                      */
/*------------------------------------------------------------------------*/

/** \fn void main(void)
 * \brief The main function of the demo.
 *
 * \todo Create description
 */
void main(void)
{


        // Initialize the Hardware and Radio
        vInitializeHW();
        lAssociatedID = 0;
        //start up in association mode
	fAssociate = TRUE;
        vHmi_ChangeLedState(eHmi_Led1_c, eHmi_LedBlink2Hz_c);
        vHmi_ChangeLedState(eHmi_Led2_c, eHmi_LedBlink2Hz_c);
        vHmi_ChangeLedState(eHmi_Led3_c, eHmi_LedBlink2Hz_c);
        vHmi_ChangeLedState(eHmi_Led4_c, eHmi_LedBlink2Hz_c);
        // start deglitching
	DeglitchRetime_Start();

        while (TRUE)
        {
                // The following Handlers requires care on invoking time interval
                if (wIsr_MsTick)
                {

                        vHmi_PbHandler();
                        vHmi_LedHandler();
                        vSampleCode_ModeChanging();
                        wIsr_MsTick = FALSE;
                }
                // Demo Application Poll-Handler function
                DemoApp_Pollhandler();
        }
}

/**
 *  Demo Application Poll-Handler
 *
 *  @note This function must be called periodically.
 *
 */
void DemoApp_Pollhandler()
{
        // if the bit buffer is full then call the packet handler function
	if ( fRetimedBitBufferFull == TRUE )
	{
	        fRetimedBitBufferFull = FALSE;
	        // call the packet handler								
	        PacketHandler ();
	}
        if ( fPacketReceived )
        {
        	fPacketReceived = FALSE;
	        if (fAssociate)
	        {
                        lAssociatedID = *(U32*)&PayloadBuffer[0];
                        fAssociate = FALSE;
                        vHmi_ChangeLedState(eHmi_Led2_c, eHmi_LedOff_c);
                        vHmi_ChangeLedState(eHmi_Led3_c, eHmi_LedOff_c);
                        vHmi_ChangeLedState(eHmi_Led4_c, eHmi_LedOff_c);
                        vHmi_ChangeLedState(eHmi_Led1_c, eHmi_LedOn_c);
                }
                else
                {
                        if (gIdIsAssociated(*(U32*)&PayloadBuffer[0]))
                        {
                                switch (PayloadBuffer[4] )
                                {
                                        case 0x01 :
                                                // LED2 and LED3 blinks once
	                                        vHmi_ChangeLedState(eHmi_Led2_c, eHmi_LedBlinkOnce_c);
	                                        vHmi_ChangeLedState(eHmi_Led3_c, eHmi_LedBlinkOnce_c);
                                                break;
                                        case 0x02 :
                                                // LED2 and LED4 blinks once
	                                        vHmi_ChangeLedState(eHmi_Led2_c, eHmi_LedBlinkOnce_c);
	                                        vHmi_ChangeLedState(eHmi_Led4_c, eHmi_LedBlinkOnce_c);
                                                break;
                                        case 0x04 :
                                                // LED2, 3, 4 blinks once
	                                        vHmi_ChangeLedState(eHmi_Led2_c, eHmi_LedBlinkOnce_c);
	                                        vHmi_ChangeLedState(eHmi_Led3_c, eHmi_LedBlinkOnce_c);
	                                        vHmi_ChangeLedState(eHmi_Led4_c, eHmi_LedBlinkOnce_c);
                                                break;
                                        case 0x05 :
                                                // LED4 blinks once
	                                        vHmi_ChangeLedState(eHmi_Led4_c, eHmi_LedBlinkOnce_c);
                                                break;
                                        case 0x09 :
                                                // LED3 blinks once
	                                        vHmi_ChangeLedState(eHmi_Led3_c, eHmi_LedBlinkOnce_c);
                                                break;
                                }
                        }
                }      
        }
}

/**
 *  Calls the init functions for the used peripherals/modules
 *
 *  @note Should be called at the beginning of the main().
 *
 */
void vInitializeHW()
{
        // Initialize the MCU peripherals
        vPlf_McuInit();

        // Initialize IO ports
        vCio_InitIO();

        // Start Timer2 peripheral with overflow interrupt
        vTmr_StartTmr2(bTmr_Tmr2Both_c, wwTmr_Tmr2Periode.U16, TRUE, bTmr_TxXCLK_00_c);


        // Start the push button handler
        vHmi_InitPbHandler();

        // Start the Led handler
        vHmi_InitLedHandler();

        // Initialize the Radio
        vRadio_Init();

        //HT6P20 OOK is set on DIPSW
        if (!SW1_A & !SW1_B )
        {
                wwTmr_Tmr3Periode.U16 = lTmr_Tmr3CntMax1_c; /**< Timer 3 period */
	        bSamplePerBitRatio = (U8)(wTmr_Tmr3DesiredFreq1_c / DATARATE1);
                bPayloadLength = PAYLOAD_BYTE_LENGTH_1;
                wBitMask = ( 1 << ( WINDOWSIZE1 - 1 ));
                bBitLimit = ( WINDOWSIZE1 / 2 );
                lSyncWordMask = ( 1L << (SYNCWORD_BIT_LENGTH1 - 1) << 1 ) - 1;
                lSyncWord = SYNCWORD1;               
                lNoiseMask = NOISEMASK_1;               
                fManchesterUsed = MANCHESTER_USED_1;
                fNormalPacket = NORMALPACKET_1;
        }
        //norm packet OOK 
        if (!SW1_A & SW1_B )
        {
                wwTmr_Tmr3Periode.U16 = lTmr_Tmr3CntMax2_c; /**< Timer 3 period */
	        bSamplePerBitRatio = (U8)(wTmr_Tmr3DesiredFreq2_c / DATARATE2);
                bPayloadLength = PAYLOAD_BYTE_LENGTH_2;
                wBitMask = ( 1 << ( WINDOWSIZE2 - 1 ));
                bBitLimit = ( WINDOWSIZE2 / 2 );
                lSyncWordMask = ( 1L << (SYNCWORD_BIT_LENGTH2 - 1) << 1 ) - 1;
                lSyncWord = SYNCWORD2;               
                lNoiseMask = NOISEMASK_2;               
                fManchesterUsed = MANCHESTER_USED_2;
                fNormalPacket = NORMALPACKET_2;
        }
        //norm packet FSK 
        if (SW1_A & SW1_B )
        {
                wwTmr_Tmr3Periode.U16 = lTmr_Tmr3CntMax3_c; /**< Timer 3 period */
	        bSamplePerBitRatio = (U8)(lTmr_Tmr3DesiredFreq3_c / DATARATE3);
                bPayloadLength = PAYLOAD_BYTE_LENGTH_3;
                wBitMask = ( 1 << ( WINDOWSIZE3 - 1 ));
                bBitLimit = ( WINDOWSIZE3 / 2 );
                lSyncWordMask = ( 1L << (SYNCWORD_BIT_LENGTH3 - 1) << 1 ) - 1;
                lSyncWord = SYNCWORD3;
                lNoiseMask = NOISEMASK_3;               
                fManchesterUsed = MANCHESTER_USED_3;
                fNormalPacket = NORMALPACKET_3;
        }

        // Start Timer3 peripheral, used to sample received data
        vTmr_StartTmr3(bTmr_Tmr3One_c, wwTmr_Tmr3Periode.U16, FALSE, bTmr_TxXCLK_00_c);
        //Set Timer3 IT to high priotity
        EIP1 |= M_PT3;

        //Initialize packet handler
        PhandlerInit();

        // Enable configured interrupts
        mIsr_EnableAllIt();
}

/** \fn void MCU_Init(void)
 *  \brief Initializes ports of the MCU.
 *
 *  \return None
 *
 *  \note It has to be called from the Initialization section.
 *  \todo Create description
 */
void vPlf_McuInit(void)
{
  volatile U32 wDelay = 0x1FFF;

  // Disable WatchDog Timer
  PCA0MD &= (~M_WDTE);

  // Init Internal Precisious Oscillator (24.5MHz)
  SFRPAGE   = LEGACY_PAGE;
  FLSCL     = 0x40;

  // Init master hw SPI interface (SCK clock: 2.45MHz)
  // Init SPI1 (Radio)
  //SPI1CFG = M_MSTEN1;     // p6: SPI1 enable master mode
  //SPI1CN  = M_SPI1EN;     // p1: SPI1 enable
  //SPI1CKR = 0x04;         // fSCK = SYSCLK / 10

  OSCICN |= M_IOSCEN;     // p7: Internal Prec. Osc. enabled
  CLKSEL  = 0x00;         // Int. Prec. Osc. selected (24.5MHz)

  P0MDIN  = 0xFF;         // All pin configured as digital port
  P1MDIN  = 0xFF;         // All pin configured as digital port
  P2MDIN  = 0xFF;         // All pin configured as digital port

  P0MDOUT = M_P0_UART_TX; // PBs: P0.0-P0.3 (same as RF_GPIO0-3) used as input
  //P1MDOUT = M_P1_SPI1_SCK | M_P1_SPI1_MOSI | M_P1_RF_NSEL | M_P1_RF_PWRDN;
  P1MDOUT = M_P1_MCU_SDN | M_P1_MCU_STBY;  
  P2MDOUT = M_P2_LED1 | M_P2_LED2 | M_P2_LED3 | M_P2_LED4 | M_P2_BZ1;

  P0SKIP  = (~M_P0_UART_TX) & (~M_P0_UART_RX); // skip all on port, but UART
  //P1SKIP  = (~M_P1_SPI1_SCK) & (~M_P1_SPI1_MISO) & (~M_P1_SPI1_MOSI); // skip all on port, but SPI1
  P1SKIP  = 0xFF;
  P2SKIP  = (~M_P2_BZ1);  // skip all on port, but buzzer with PCA CEX0

  SFRPAGE = CONFIG_PAGE;
  P0DRV   = 0x00;
  P1DRV   = 0x00;
  P2DRV   = 0x00;

  SFRPAGE = LEGACY_PAGE;
  XBR0    = M_URT0E;      // p0: UART enabled on XBAR
  //XBR1    = M_SPI1E | (1u << BF_PCA0ME_0); // p6: SPI1 enabled on XBAR, p0: PCA CEX0 enabled on XBAR
  XBR1 = ( 1 << BF_PCA0ME_0);
  XBR2    = M_XBARE;      // p6: XBAR enable

  // latch all inputs to '1'
  P0 = ~P0MDOUT;
  P1 = ~P1MDOUT;
  P2 = ~P2MDOUT;

  // set all output to its default state

  LED1 = ILLUMINATE;
  LED2 = ILLUMINATE;
  LED3 = ILLUMINATE;
  LED4 = ILLUMINATE;
  
  for (wDelay = 0x1FFF; wDelay--; )  ;
    
  LED1 = EXTINGUISH;
  LED2 = EXTINGUISH;
  LED3 = EXTINGUISH;
  LED4 = EXTINGUISH;

  BZ1 = 1;
  //RF_NSEL = TRUE;
  //RF_PWRDN = FALSE;

  /* Startup delay */
  for (wDelay = 0x1FFF; wDelay--; )  ;
}

#ifdef SDCC
/**
 * \brief External startup function of SDCC.
 *
 * It performs operations
 * prior static and global variable initialization.
 * Watchdog timer should be disabled this way, otherwise it
 * can expire before variable initialization is carried out,
 * and may prevent program execution jumping into main().
 *
 * \param None
 * \return None
 */
void _sdcc_external_startup(void)
{
  PCA0MD &= ~0x40;      // Disable Watchdog timer
}
#endif
