/*! @file control_IO.c
 * @brief This file contains functions for controlling LEDs and PBs.
 *
 * @b COPYRIGHT
 * @n Silicon Laboratories Confidential
 * @n Copyright 2012 Silicon Laboratories, Inc.
 * @n http://www.silabs.com
 */

#include "..\drivers\bsp.h"

/*------------------------------------------------------------------------*/
/*                          Global variables                              */
/*------------------------------------------------------------------------*/

SEGMENT_VARIABLE(bCio_IoPortState, U8, SEG_DATA); /**< State holder of IO port*/


/*------------------------------------------------------------------------*/
/*                          Function implementations                      */
/*------------------------------------------------------------------------*/

/*!
 * This function is used to initialize specific IO port for LED & PB.
 *
 *  @return None
 *
 *  @note It has to be called from the initialization section.
 */
void vCio_InitIO(void)
{

#ifdef SILABS_PLATFORM_RFSTICK
  bCio_IoPortState = (~M_BZ1);    // Config all to '1', but buzzer
  P2 = bCio_IoPortState;
#else
#error TO BE WRITTEN FOR OTHER PLARFORMS!
#endif
}

/*!
 * This function is used to switch the selected LED on.
 *
 * @param[in] biLedNum  Number of the LED to be switched on (1 ... 4).
 *
 * @return None
 */
void vCio_SetLed(U8 biLedNum)
{
  switch(biLedNum)
  {
    case 1:
      LED1 = ILLUMINATE;
      bCio_IoPortState &= (~M_LED1);
      break;
    case 2:
      LED2 = ILLUMINATE;
      bCio_IoPortState &= (~M_LED2);
      break;
    case 3:
      LED3 = ILLUMINATE;
      bCio_IoPortState &= (~M_LED3);
      break;
    case 4:
      LED4 = ILLUMINATE;
      bCio_IoPortState &= (~M_LED4);
      break;
    default:
      break;
  }
}

/*!
 * This function is used to switch the selected LED off.
 *
 * @param[in] biLedNum Number of the LED to be switched off (1 ... 4).
 *
 * @return None
 */
void vCio_ClearLed(U8 biLedNum)
{
  switch(biLedNum)
  {
    case 1:
      LED1 = EXTINGUISH;
      bCio_IoPortState |= M_LED1;
      break;
    case 2:
      LED2 = EXTINGUISH;
      bCio_IoPortState |= M_LED2;
      break;
    case 3:
      LED3 = EXTINGUISH;
      bCio_IoPortState |= M_LED3;
      break;
    case 4:
      LED4 = EXTINGUISH;
      bCio_IoPortState |= M_LED4;
      break;
    default:
      break;
  }
}

/*!
 * This function is used to read the status of the selected push-button.
 *
 * @param[in] biPbNum Number of the push-button to be switched on (1 ... 4).
 *
 * @return State of the selected PB.
 */
BIT gCio_GetPB(U8 biPbNum)
{

#ifdef SILABS_PLATFORM_RFSTICK
  BIT s;

  //Disable Global interrupt
  mIsr_DisableAllIt();

  switch (biPbNum)
  {
    case 1u:
      s = PB1;
      break;

    case 2u:
      s = PB2;
      break;

    case 3u:
      s = PB3;
      break;

    case 4u:
      s = PB4;
      break;

    default:
      s = TRUE;
      break;
  }

  //Enable Global interrupt
  mIsr_EnableAllIt();

  //return with the value of the button
  return s;

#else
  BIT s;

#warning First compile warning: Check your platform if it can support these settings.

  //check whether the selected button was pressed or not
  switch (biPbNum)
  {
    case 1u:
      // Disable Global interrupt
      mIsr_DisableAllIt();
      // latch P2.x to '1'
      LED1 = EXTINGUISH;
      //set P2.x to input
      P2MDOUT &= (~M_LED1);
      NOP();
      s = LED1;
      //set P2.x to output
      P2MDOUT |= M_LED1;
      //restore LED1
      if ((bCio_IoPortState & M_LED1) == 0u)
        SetLed(1u);
      else
        ClearLed(1u);
      //Enable Global interrupt
      mIsr_EnableAllIt();
      //return with the value of the button
    break;

    case 2u:
      // Disable Global interrupt
      mIsr_DisableAllIt();
      // latch P2.x to '1'
      LED2 = EXTINGUISH;
      //set P2.x to input
      P2MDOUT &= (~M_LED2);
      NOP();
      s = LED2;
      //set P2.x to output
      P2MDOUT |= M_LED2;
      //restore LED2
      if((bCio_IoPortState & M_LED2) == 0u)
        SetLed(2);
      else
        ClearLed(2);
      //Enable Global interrupt
      mIsr_EnableAllIt();
      //return with the value of the button
    break;

    case 3u:
      //Disable Global interrupt
      mIsr_DisableAllIt();
      //latch P2.x to '1'
      LED3 = EXTINGUISH;
      //set P2.x to input
      P2MDOUT &= (~M_LED3);
      NOP();
      s = LED3;
      //set P2.x to output
      P2MDOUT |= M_LED3;
      //restore LED3
      if((bCio_IoPortState & M_LED3) == 0u)
        SetLed(3);
      else
        ClearLed(3);
      //Enable Global interrupt
      mIsr_EnableAllIt();
      //return with the value of the button
    break;

    case 4:
      //Disable Global interrupt
      mIsr_DisableAllIt();
      //latch P2.x to '1'
      LED4 = EXTINGUISH;
      //set P2.0 to input
      P2MDOUT &= (~M_LED4);
      NOP();
      s = LED4;
      //set P2.x to output
      P2MDOUT |= M_LED4;
      //restore LED4
      if((bCio_IoPortState & M_LED4) == 0u)
        SetLed(4);
      else
        ClearLed(4);
      //Enable Global interrupt
      mIsr_EnableAllIt();
      //return with the value of the button
    break;

    default:
      s = TRUE;
    break;
  }

  return s;
#endif
}
