/*! @file menu.c
 * @brief This file contains functions of the menu handler driver.
 *
 * @b COPYRIGHT
 * @n Silicon Laboratories Confidential
 * @n Copyright 2012 Silicon Laboratories, Inc.
 * @n http://www.silabs.com
 */

#include "..\drivers\bsp.h"

/*------------------------------------------------------------------------*/
/*                          Global variables                              */
/*------------------------------------------------------------------------*/

SEGMENT_VARIABLE(rMnu_MenuStateInfo, tMnu_MenuStateInfo, SEG_XDATA); /**< This instance of the structure holds information of information of menu states */

/*!
 * @brief Pointer to the switch state handler function of the application.
 * @note It have to be implemented in the application menu files.
 */
U8 (*pbMnu_AppMnuSwStateHandler)(void) = NULL;

/*!
 * @brief Pointer to an instance of the tMnu_MenuState structure, which is stored on the code space.
 * @note It have to be implemented in the application menu files.
 */
SEGMENT_VARIABLE_SEGMENT_POINTER(parMnu_AppMenuState, tMnu_MenuState, SEG_CODE, SEG_XDATA);

/*!
 * @brief Pointer to an instance of the tMnu_MenuNextState structure, which is stored on the code space.
 * @note It have to be implemented in the application menu files.
 */
SEGMENT_VARIABLE_SEGMENT_POINTER(parMnu_AppMenuNextState, tMnu_MenuNextState, SEG_CODE, SEG_XDATA);


/*------------------------------------------------------------------------*/
/*                          Function implementations                      */
/*------------------------------------------------------------------------*/

/*!
 * This function is used to handle switch state change.
 *
 * @param[in] pariAppMenuState Pointer to the instance of the tMnu_MenuState structure.
 * @param[in] pariAppMenuNextState Pointer to the instance of the tMnu_MenuNextState structure.
 * @param[in] pbiAppMnuSwStateHandler Pointer to the switch state handler function of the application.
 *
 * @return  True if state of switches has changed.
 *
 * @note It has to be called from the initialization section.
 */
void vMnu_InitMenu(tMnu_MenuState *pariAppMenuState, tMnu_MenuNextState *pariAppMenuNextState, U8 (*pbiAppMnuSwStateHandler)(void))
{
  parMnu_AppMenuState         = pariAppMenuState;
  parMnu_AppMenuNextState     = pariAppMenuNextState;
#ifdef SILABS_PLATFORM_COMPONENT_SWITCH
  pbMnu_AppMnuSwStateHandler  = pbiAppMnuSwStateHandler;
#else
  pbMnu_AppMnuSwStateHandler  = NULL;
#endif

  rMnu_MenuStateInfo.bMenuState       = parMnu_AppMenuState[0].bMenuState;
  rMnu_MenuStateInfo.bMenuNextState   = parMnu_AppMenuState[0].bMenuState;
  rMnu_MenuStateInfo.pbStateFunction  = parMnu_AppMenuState[0].pStateFunction;
  rMnu_MenuStateInfo.bInput           = eHmi_PbNo_c;
}

/*!
 * This function is used to manage menu states and the transitions from one state to an other.
 *
 * @return None.
 */
void vMnu_MenuHandler(void)
{

  if (gHmi_SwStateHandler())
  {
    if (pbMnu_AppMnuSwStateHandler)
    {
      rMnu_MenuStateInfo.bMenuNextState = pbMnu_AppMnuSwStateHandler();
    }
  }
  else
  {
    if (gHmi_IsPbUnHandled())
    {
      rMnu_MenuStateInfo.bInput = bHmi_PbGetLastButton(0);
    }
    else
    {
      rMnu_MenuStateInfo.bInput = eHmi_PbNo_c;
    }

    if (rMnu_MenuStateInfo.pbStateFunction)
    {
      rMnu_MenuStateInfo.bMenuNextState = rMnu_MenuStateInfo.pbStateFunction(rMnu_MenuStateInfo.bInput);
    }
    else if (rMnu_MenuStateInfo.bInput)
    {
      rMnu_MenuStateInfo.bMenuNextState = bMnu_MenuButtonStep(rMnu_MenuStateInfo.bMenuState, rMnu_MenuStateInfo.bInput);
    }
  }

  if (rMnu_MenuStateInfo.bMenuNextState != rMnu_MenuStateInfo.bMenuState)
  {
    vMnu_MenuChangeState(rMnu_MenuStateInfo.bMenuNextState);
  }

}

/*!
 * This function is used to change the menu state.
 *
 * @param[in] biNewMenuState New menu state.
 *
 * @return None.
 */
void vMnu_MenuChangeState(U8 biNewMenuState)
{
  SEGMENT_VARIABLE(bMnuCnt, U8, SEG_DATA);

  rMnu_MenuStateInfo.bMenuState = biNewMenuState;
  for (bMnuCnt = 0; parMnu_AppMenuState[bMnuCnt].bMenuState; bMnuCnt++)
  {
    if (parMnu_AppMenuState[bMnuCnt].bMenuState == rMnu_MenuStateInfo.bMenuState)
    {
      rMnu_MenuStateInfo.pbStateFunction = parMnu_AppMenuState[bMnuCnt].pStateFunction;
      break;
    }
  }
}

/*!
 * This function is used to get menu state transition caused by buttons.
 *
 * @param[in] biMenuState Actual menu state.
 * @param[in] biInput Stimuli of state transition.
 *
 * @return Next menu state after the transition or same if no such transition.
 */
U8 bMnu_MenuButtonStep(U8 biMenuState, U8 biInput)
{
  SEGMENT_VARIABLE(bNextState,  U8, SEG_DATA) = biMenuState; /**< Default stay in same state */
  SEGMENT_VARIABLE(bMnuCnt,     U8, SEG_DATA);

  // Step through the next state array (note: last element is NULL)
  for (bMnuCnt = 0; parMnu_AppMenuNextState[bMnuCnt].bMenuState; bMnuCnt++)
  {
    if ((parMnu_AppMenuNextState[bMnuCnt].bMenuState == biMenuState) && (parMnu_AppMenuNextState[bMnuCnt].bInput == biInput))
    {
      // This is the one!
      bNextState = parMnu_AppMenuNextState[bMnuCnt].bMenuNextState;
      break;
    }
  }

  return bNextState;
}

/*!
 * This function is used to clear handler function of a menu state.
 * It can be used if handler required only for first time and all the transitions can be managed by the menu handler.
 *
 * @return None.
 */
void vMnu_DeleteMenuFunction(void)
{
  rMnu_MenuStateInfo.pbStateFunction = NULL;
}

/*!
 * This function is used to get the actual menu state.
 *
 * @return Actual menu state.
 */
U8 bMnu_GetMenuState(void)
{
  return rMnu_MenuStateInfo.bMenuState;
}


