/*! @file spi.c
 * @brief This file contains functions for controlling SPI0 and SPI1 interfaces.
 *
 * @b COPYRIGHT
 * @n Silicon Laboratories Confidential
 * @n Copyright 2012 Silicon Laboratories, Inc.
 * @n http://www.silabs.com
 */


#include "..\drivers\bsp.h"


/*------------------------------------------------------------------------*/
/*                          Global variables                              */
/*------------------------------------------------------------------------*/

#ifdef SILABS_MCU_DC_MLCD
SEGMENT_VARIABLE(bSpi_SelectStateSpi0, U8 , SEG_DATA); /**< Current state of the NSEL of SPI0 */
BIT gSpi_Spi0Disabled;  /**< Current state of SPI0 */
#endif

SEGMENT_VARIABLE(bSpi_SelectStateSpi1, U8 , SEG_DATA); /**< Current state of the NSEL of SPI1 */
BIT gSpi_Spi1Disabled;  /**< Current state of SPI1 */


/*------------------------------------------------------------------------*/
/*                          Function implementations                      */
/*------------------------------------------------------------------------*/

#ifdef SILABS_MCU_DC_MLCD
/*!
 * This function is used to read/write one byte from/to SPI0.
 *
 * @param[in] biDataIn    Data to be sent.
 * @return  Read value of the SPI port after writing on it.
 */
U8 bSpi_ReadWriteSpi0(U8 biDataIn)
{
  U8 bValue;
  BIT gRestoreEA;

#ifdef SILABS_MCU_F960
  SEGMENT_VARIABLE(bRestoreSfrPage, U8, SEG_DATA);
#endif

  // disable interrupts during SPI transfer
  gRestoreEA = EA;
  EA = 0;

#ifdef SILABS_MCU_F960
  // save SFR page and switch to SPI1 page
  bRestoreSfrPage = SFRPAGE;
  SFRPAGE = LEGACY_PAGE ;
#endif

  // Send SPI data using double buffered write
  SPIF0 = 0;                          // clear SPIF
  SPI0DAT = ( biDataIn );                  // write data register
  while(!TXBMT0);                     // wait on TXBMT
  while((SPI0CFG & M_SPIBSY0) == M_SPIBSY0);    // wait on SPIBSY
  bValue = SPI0DAT;                    // read value
  SPIF0 = 0;                          // leave SPIF cleared

#ifdef SILABS_MCU_F960
  // Restore SFR page after SPI transfer
  SFRPAGE = bRestoreSfrPage;
#endif

  // Restore interrupts after SPI transfer
  EA = gRestoreEA;

  return bValue;
}
#endif

/*!
 * This function is used to read/write one byte from/to SPI1.
 *
 * @param[in] biDataIn    Data to be sent.
 * @return  Read value of the SPI port after writing on it.
 */
U8 bSpi_ReadWriteSpi1(U8 biDataIn)
{  
  U8 bValue;
  BIT gRestoreEA;

#ifdef SILABS_MCU_F960
  SEGMENT_VARIABLE(bRestoreSfrPage, U8, SEG_DATA);
#endif

  // disable interrupts during SPI transfer
  gRestoreEA = EA;
  EA = 0;

#ifdef SILABS_MCU_F960
  // save SFR page and switch to SPI1 page
  bRestoreSfrPage = SFRPAGE;
  SFRPAGE = LEGACY_PAGE ;
#endif

  // Send SPI data using double buffered write
  SPIF1 = 0;                          // clear SPIF
  SPI1DAT = ( biDataIn );                  // write data register
  while(!TXBMT1);                     // wait on TXBMT
  while((SPI1CFG & M_SPIBSY1) == M_SPIBSY1);    // wait on SPIBSY
  bValue = SPI1DAT;                    // read value
  SPIF1 = 0;                          // leave SPIF cleared

#ifdef SILABS_MCU_F960
  // Restore SFR page after SPI transfer
  SFRPAGE= bRestoreSfrPage;
#endif

  // Restore interrupts after SPI transfer
  EA = gRestoreEA;

  return bValue;
}



#ifdef SILABS_MCU_DC_MLCD
/*!
 * This function is used to send data over SPI0 no response expected.
 *
 *  @param[in] biDataInLength  The length of the data.
 *  @param[in] *pabiDataIn     Pointer to the first element of the data.
 *
 *  @return None
 */
void vSpi_WriteDataSpi0(U8 biDataInLength, U8 *pabiDataIn)
{
  SEGMENT_VARIABLE(bCnt, U8, SEG_XDATA);

  for (bCnt=0; bCnt<biDataInLength; bCnt++)    // Send input data array via SPI
  {
    bSpi_ReadWriteSpi0(pabiDataIn[bCnt]);
  }
}
#endif

/*!
 * This function is used to send data over SPI1 no response expected.
 *
 *  @param[in] biDataInLength  The length of the data.
 *  @param[in] *pabiDataIn     Pointer to the first element of the data.
 *
 *  @return None
 */
void vSpi_WriteDataSpi1(U8 biDataInLength, U8 *pabiDataIn)
{
  SEGMENT_VARIABLE(bCnt, U8, SEG_XDATA);

#ifdef SILABS_MCU_F960
  #warning Check if implemenations of DMA makes a sense for SPI on F960!
#endif

  for (bCnt=0; bCnt<biDataInLength; bCnt++)    // Send input data array via SPI
  {
    bSpi_ReadWriteSpi1(pabiDataIn[bCnt]);
  }
}

#ifdef SILABS_MCU_DC_MLCD
/*!
 * This function is used to read data from SPI0.
 *
 *  \param[in] biDataOutLength  The length of the data.
 *  \param[out] *paboDataOut    Pointer to the first element of the response.
 *
 *  \return None
 */
void vSpi_ReadDataSpi0(U8 biDataOutLength, U8 *paboDataOut)
{
  SEGMENT_VARIABLE(bCnt, U8, SEG_XDATA);

  // send command and get response from the radio IC
  for (bCnt=0; bCnt<biDataOutLength; bCnt++)
  {
    paboDataOut[bCnt] = bSpi_ReadWriteSpi0(0xFF);            // Store data byte that came from the radio IC
  }
}
#endif

/*!
 * This function is used to read data from SPI1.
 *
 *  \param[in] biDataOutLength  The length of the data.
 *  \param[out] *paboDataOut    Pointer to the first element of the response.
 *
 *  \return None
 */
void vSpi_ReadDataSpi1(U8 biDataOutLength, U8 *paboDataOut)
{
  SEGMENT_VARIABLE(bCnt, U8, SEG_XDATA);

#ifdef SILABS_MCU_F960
  #warning Check if implemenations of DMA makes a sense for SPI on F960!
#endif

  // send command and get response from the radio IC
  for (bCnt=0; bCnt<biDataOutLength; bCnt++)
  {
    paboDataOut[bCnt] = bSpi_ReadWriteSpi1(0xFF);            // Store data byte that came from the radio IC
  }
}


#ifdef SPI_DRIVER_EXTENDED_SUPPORT
  #ifdef SILABS_MCU_DC_MLCD
  /*!
   * This function is used to enable SPI0 and associate to XBAR.
   *
   * @return  None.
   */
  void vSpi_EnableSpi0(void)
  {
  #ifdef SILABS_MCU_F960
    SEGMENT_VARIABLE(bRestoreSfrPage, U8, SEG_DATA);
  #endif

  #ifdef SILABS_MCU_F960
    // save SFR page and switch to SPI1 page
    bRestoreSfrPage= SFRPAGE;
    SFRPAGE = LEGACY_PAGE;
  #endif

    //enable SPI port
    SPI0CN |= M_SPI0EN;

    //associate to the XBAR
    XBR0 |= M_SPI0E;
    //configure IO pins
    P0SKIP  &= (~M_P0_SPI0_SCK) & (~M_P0_SPI0_MISO) & (~M_P0_SPI0_MOSI);
    //set default state for pins
    SPI0_SCK = 1;
    SPI0_MOSI = 1;
    gSpi_Spi0Disabled = 0;

  #ifdef SILABS_MCU_F960
    // Restore SFR page after SPI transfer
    SFRPAGE = bRestoreSfrPage;
  #endif
  }
  #endif

  /*!
   * This function is used to enable SPI1 and associate to XBAR.
   *
   * @return  None.
   */
  void vSpi_EnableSpi1(void)
  {
  #ifdef SILABS_MCU_F960
    SEGMENT_VARIABLE(bRestoreSfrPage, U8, SEG_DATA);
  #endif

  #ifdef SILABS_MCU_F960
    // save SFR page and switch to SPI1 page
    bRestoreSfrPage= SFRPAGE;
    SFRPAGE = SPI1_PAGE;
  #endif

    //enable SPI port
    SPI1CN |= M_SPI1EN;

  #ifdef SILABS_MCU_F960
    SFRPAGE = LEGACY_PAGE;
  #endif

    //associate to the XBAR
    XBR1 |= M_SPI1E;
    //configure IO pins
    P1SKIP  &= (~M_P1_SPI1_SCK) & (~M_P1_SPI1_MISO) & (~M_P1_SPI1_MOSI);
    //set default state for pins
    SPI1_SCK = 1;
    SPI1_MOSI = 1;
    gSpi_Spi1Disabled = 0;

  #ifdef SILABS_MCU_F960
    // Restore SFR page after SPI transfer
    SFRPAGE = bRestoreSfrPage;
  #endif
  }

  #ifdef SILABS_MCU_DC_MLCD
  /*!
   * This function is used to disable SPI0 and disconnect from XBAR.
   *
   * @return  None.
   */
  void vSpi_DisableSpi0(void)
  {
  #ifdef SILABS_MCU_F960
    SEGMENT_VARIABLE(bRestoreSfrPage, U8, SEG_DATA);
  #endif

  #ifdef SILABS_MCU_F960
    // save SFR page and switch to SPI1 page
    bRestoreSfrPage= SFRPAGE;
    SFRPAGE = LEGACY_PAGE;
  #endif

    //disable SPI port
    SPI0CN &= (~M_SPI0EN);

    //disconnect from XBAR
    XBR0 &= (~M_SPI0E);
    //configure IO port directions
    P0SKIP  |= M_P0_SPI0_SCK | M_P0_SPI0_MISO | M_P0_SPI0_MOSI;
    //set default state for the pins
    SPI0_SCK = 0;
    SPI0_MOSI = 1;
    gSpi_Spi0Disabled = 1;

  #ifdef SILABS_MCU_F960
    // Restore SFR page after SPI transfer
    SFRPAGE = bRestoreSfrPage;
  #endif
  }
  #endif

  /*!
   * This function is used to disable SPI1 and disconnect from XBAR.
   *
   * @return  None.
   */
  void vSpi_DisableSpi1(void)
  {
  #ifdef SILABS_MCU_F960
    SEGMENT_VARIABLE(bRestoreSfrPage, U8, SEG_DATA);
  #endif

  #ifdef SILABS_MCU_F960
    // save SFR page and switch to SPI1 page
    bRestoreSfrPage= SFRPAGE;
    SFRPAGE = SPI1_PAGE;
  #endif

    //disable SPI port
    SPI1CN &= (~M_SPI1EN);

  #ifdef SILABS_MCU_F960
    SFRPAGE = LEGACY_PAGE;
  #endif

    //disconnect from XBAR
    XBR1 &= (~M_SPI1E);
    //configure IO port directions
    P1SKIP  |= M_P1_SPI1_SCK | M_P1_SPI1_MISO | M_P1_SPI1_MOSI;
    //set default state for the pins
    SPI1_SCK = 0;
    SPI1_MOSI = 1;
    gSpi_Spi1Disabled = 1;

  #ifdef SILABS_MCU_F960
    // Restore SFR page after SPI transfer
    SFRPAGE = bRestoreSfrPage;
  #endif
  }

  #ifdef SILABS_MCU_DC_MLCD
  /*!
   * This function is used to pull down nSEL of the selected device on SPI0.
   *
   * @param[in] biSelectDevice Selected device
   *
   *  @return  None.
   */
  void vSpi_ClearNselSpi0(U8 biSelectDevice)
  {

  #error SPI0 nselect is not implemented yet!

    bSpi_SelectStateSpi0 = 0;

    switch(biSelectDevice)
    {
      case 0:
        break;

      default:
      break;
    }
  }
  #endif

  /*!
   * This function is used to pull down nSEL of the selected device on SPI1.
   *
   * @param[in] biSelectDevice Selected device
   *  - 0 - DUT
   *  - 2 - EEPROM
   *  - 3 - MCU2
   *
   *  @return  None.
   */
  void vSpi_ClearNselSpi1(U8 biSelectDevice)
  {

    bSpi_SelectStateSpi1 = 0;

    switch(biSelectDevice)
    {
      case 0:
        RF_NSEL = 0;
        break;

  #ifdef SILABS_PLATFORM_DKMB
      case 1:
      break;

      case 2:
          EENSEL = 0;
      break;

      case 3:
          PEXT_NSEL = 0;
      break;
  #endif

      default:
      break;
    }
  }

  #ifdef SILABS_MCU_DC_MLCD
  /*!
   * This function is used to pull up nSEL of the selected device on SPI0.
   *
   * @param[in] biSelectDevice Selected device
   *
   *  @return  None.
   */
  void vSpi_SetNselSpi0(U8 biSelectDevice)
  {

  #error SPI0 nselect is not implemented yet!

    bSpi_SelectStateSpi0 = 1;

    switch(biSelectDevice)
    {
      case 0:
        break;

      default:
        break;
    }
  }
  #endif

  /*!
   * This function is used to pull up nSEL of the selected device on SPI1.
   *
   * @param[in] biSelectDevice Selected device
   *  - 0 - DUT
   *  - 2 - EEPROM
   *  - 3 - MCU2
   *
   *  @return  None.
   */
  void vSpi_SetNselSpi1(U8 biSelectDevice)
  {

    bSpi_SelectStateSpi1 = 1;

    switch(biSelectDevice)
    {
      case 0:
        RF_NSEL = 1;
        break;

  #ifdef SILABS_PLATFORM_DKMB
      case 1:
      break;

      case 2:
          EENSEL = 0;
      break;

      case 3:
          PEXT_NSEL = 0;
      break;
  #endif

      default:
        break;
    }
  }

  #ifdef SILABS_MCU_DC_MLCD
  /*!
   * This function is used to read one byte from SPI0 using bitbang method.
   *
   * @return Read byte
   */
  U8 bSpi_ReadByteBitbangSpi0(void)
  {
      SEGMENT_VARIABLE(bReadByte, U8, SEG_DATA);
      SEGMENT_VARIABLE(bCnt, U8, SEG_DATA);

      bReadByte = 0;
      SPI0_SCK = 1;
      NOP();
      NOP();
      SPI0_SCK = 0;
      NOP();
      for(bCnt = 0; bCnt < 8; bCnt++)
      {
          SPI0_SCK = 1;
          bReadByte <<= 1;
          //take sample
          if(SPI0_MISO == 1)
          {
            bReadByte |= 0x01;
          }
          SPI0_SCK = 0;
      }

      return bReadByte;
  }
  #endif

  /*!
   * This function is used to read one byte from SPI1 using bitbang method.
   *
   * @return Read byte
   */
  U8 bSpi_ReadByteBitbangSpi1(void)
  {
      SEGMENT_VARIABLE(bReadByte, U8, SEG_DATA);
      SEGMENT_VARIABLE(bCnt, U8, SEG_DATA);

      bReadByte = 0;
      SPI1_SCK = 1;
      NOP();
      NOP();
      SPI1_SCK = 0;
      NOP();
      for(bCnt = 0; bCnt < 8; bCnt++)
      {
          SPI1_SCK = 1;
          bReadByte <<= 1;
          //take sample
          if(SPI1_MISO == 1)
          {
            bReadByte |= 0x01;
          }
          SPI1_SCK = 0;
      }

      return bReadByte;
  }

  #ifdef SILABS_MCU_DC_MLCD
  /*!
   * This function is used to write specified number of bits to SPI0 using bitbang method.
   *
   * @param[in] biDataIn Input byte of data bits
   * @param[out] biNumOfBits Number of bits to be written to SPI
   *
   * @return None
   */
  void vSpi_WriteBitsBitbangSpi0(U8 biDataIn, U8 biNumOfBits)
  {
      SEGMENT_VARIABLE(bCnt, U8, SEG_DATA);
      SEGMENT_VARIABLE(bMask, U8, SEG_DATA);

      bMask = (0x01 << (biNumOfBits-1));
      for(bCnt = 0; bCnt < biNumOfBits; bCnt++)
      {
          SPI0_SCK = 0;
          if((biDataIn & bMask) != 0)
          {
              SPI0_MOSI = 1;
          }
          else
          {
              SPI0_MOSI = 0;
          }
          //clock pulse to sample
          SPI0_SCK = 1;
          bMask >>= 1;
      }
      SPI0_SCK = 0;
  }
  #endif

  /*!
   * This function is used to write specified number of bits to SPI1 using bitbang method.
   *
   * @param[in] biDataIn Input byte of data bits
   * @param[out] biNumOfBits Number of bits to be written to SPI
   *
   * @return None
   */
  void vSpi_WriteBitsBitbangSpi1(U8 biDataIn, U8 biNumOfBits)
  {
      SEGMENT_VARIABLE(bCnt, U8, SEG_DATA);
      SEGMENT_VARIABLE(bMask, U8, SEG_DATA);

      bMask = (0x01 << (biNumOfBits-1));
      for(bCnt = 0; bCnt < biNumOfBits; bCnt++)
      {
          SPI1_SCK = 0;
          if((biDataIn & bMask) != 0)
          {
              SPI1_MOSI = 1;
          }
          else
          {
              SPI1_MOSI = 0;
          }
          //clock pulse to sample
          SPI1_SCK = 1;
          bMask >>= 1;
      }
      SPI1_SCK = 0;
  }

#endif
