/*! @file timer.c
 * @brief This file contains the interface functions of the timers.
 *
 * @b COPYRIGHT
 * @n Silicon Laboratories Confidential
 * @n Copyright 2012 Silicon Laboratories, Inc.
 * @n http://www.silabs.com
 */

#include "..\drivers\bsp.h"

/*------------------------------------------------------------------------*/
/*                          Global variables                              */
/*------------------------------------------------------------------------*/

SEGMENT_VARIABLE(wwTmr_Tmr2Periode , UU16, SEG_DATA) = {lTmr_Tmr2CntMax_c}; /**< Timer 2 period (1ms) */


/*------------------------------------------------------------------------*/
/*                          Function implementations                      */
/*------------------------------------------------------------------------*/

/*!
 * This function is used to start Timer 2 in the specified mode.
 *
 * @param[in] biPrescaler Prescaler value of timer (use predefined constants: bTmr_Tmr2One_c, bTmr_Tmr2Both_c)
 * @param[in] wwiPeriod   The duration of the timing
 * @param[in] biItEnable  Enables timer IT if TRUE, disables it if FALSE
 * @param[in] biExtClkSel External clock select (use predefined constants: bTmr_TxXCLK_00_c etc.)
 *
 * @return  None.
 */
void vTmr_StartTmr2(U8 biPrescaler, U16 wiPeriod, U8 biItEnable, U8 biExtClkSel)
{
  SEGMENT_VARIABLE(wwiPeriod , UU16, SEG_DATA);
  wwiPeriod.U16 = wiPeriod;

  //setup timer
  CKCON = (CKCON & (~(M_T2MH | M_T2ML))) | biPrescaler;
  TMR2CN = biExtClkSel & M_T2XCLK;

  //set time period
  wwiPeriod.U16 = 65535 - wwiPeriod.U16;
  TMR2H = wwiPeriod.U8[MSB];
  TMR2L = wwiPeriod.U8[LSB];
  TMR2RLH = wwiPeriod.U8[MSB];
  TMR2RLL = wwiPeriod.U8[LSB];

  //enable IT if needed
  if(biItEnable)
  {
    mTmr_EnableTmr2It();
  }
  else
  {
    mTmr_DisableTmr2It();
  }
  mTmr_EnableTmr2();
}

#ifdef TIMER_DRIVER_EXTENDED_SUPPORT
/*!
 * This function is used to check if Timer 2 is expired.
 *
 * @return  True if timer is expired (also stops the timer).
 * @note Function clears the IT status flag as well.
 */
BIT gTmr_Tmr2Expired(void)
{
  if( TF2H == 0)
  {
    return FALSE;
  }
  else
  {
    mTmr_StopTmr2();
    return TRUE;
  }

}

/*!
 * This function is used to start Timer 3 in the specified mode.
 *
 * @param[in] biPrescaler Prescaler value of timer (use predefined constants: bTmr_Tmr3One_c, bTmr_Tmr3Both_c)
 * @param[in] wwiPeriod   The duration of the timing
 * @param[in] biItEnable  Enables timer IT if TRUE, disables it if FALSE
 * @param[in] biExtClkSel External clock select (use predefined constants: bTmr_TxXCLK_00_c etc.)
 *
 * @return  None.
 */
void vTmr_StartTmr3(U8 biPrescaler, U16 wiPeriod, U8 biItEnable, U8 biExtClkSel)
{
  SEGMENT_VARIABLE(wwiPeriod , UU16, SEG_DATA);
  wwiPeriod.U16 = wiPeriod;

#ifdef SILABS_MCU_F960
  SEGMENT_VARIABLE(bRestoreSfrPage, U8, SEG_DATA);

  bRestoreSfrPage = SFRPAGE;
  SFRPAGE = LEGACY_PAGE;
#endif

  //setup timer
  CKCON = (CKCON & (~(M_T3MH | M_T3ML))) | biPrescaler;
  TMR3CN = biExtClkSel & M_T3XCLK;

  //set time period
  wwiPeriod.U16 = 65535 - wwiPeriod.U16;
  TMR3H = wwiPeriod.U8[MSB];
  TMR3L = wwiPeriod.U8[LSB];
  TMR3RLH = wwiPeriod.U8[MSB];
  TMR3RLL = wwiPeriod.U8[LSB];

  //enable IT if needed
  if( biItEnable == 1 )
  {
    mTmr_EnableTmr3It();
  }
  else
  {
    mTmr_DisableTmr3It();
  }
  mTmr_EnableTmr3();

#ifdef SILABS_MCU_F960
  // Restore SFR page after SPI transfer
  SFRPAGE = bRestoreSfrPage;
#endif
}

/*!
 * This function is used to check if Timer 3 is expired.
 *
 * @return  True if timer is expired (also stops the timer).
 * @note Function clears the IT status flag as well.
 */
BIT gTmr_Tmr3Expired(void)
{
#ifdef SILABS_MCU_F960
  SEGMENT_VARIABLE(bRestoreSfrPage, U8, SEG_DATA);

  bRestoreSfrPage = SFRPAGE;
  SFRPAGE = LEGACY_PAGE;
#endif

  if( (TMR3CN & M_TF3H) == 0)
  {
    return 0;
  }
  else
  {
    mTmr_StopTmr3();
    return 1;
  }

#ifdef SILABS_MCU_F960
  // Restore SFR page after SPI transfer
  SFRPAGE = bRestoreSfrPage;
#endif
}

#endif
